@@ -29,7 +29,7 @@ from ramalama.config import CONFIG
 from ramalama.logger import configure_logger, logger
 from ramalama.model import MODEL_TYPES
 from ramalama.model_factory import ModelFactory, New
-from ramalama.model_store import GlobalModelStore
+from ramalama.model_store.global_store import GlobalModelStore
 from ramalama.shortnames import Shortnames
 from ramalama.stack import Stack
 from ramalama.version import print_version, version
@@ -148,6 +148,10 @@ with software on the local system.
 """
 
 
+def abspath(astring) -> str:
+    return os.path.abspath(astring)
+
+
 def create_argument_parser(description: str):
     """Create and configure the argument parser for the CLI."""
     parser = ArgumentParserWithDefaults(
@@ -191,7 +195,7 @@ The RAMALAMA_CONTAINER_ENGINE environment variable modifies default behaviour.""
     )
     parser.add_argument(
         "--image",
-        default=accel_image(CONFIG, pull=False),
+        default=accel_image(CONFIG),
         help="OCI container image to run with the specified AI model",
         action=OverrideDefaultAction,
         completer=local_images,
@@ -216,12 +220,13 @@ The RAMALAMA_IN_CONTAINER environment variable modifies default behaviour.""",
     parser.add_argument(
         "--runtime",
         default=CONFIG.runtime,
-        choices=["llama.cpp", "vllm"],
-        help="specify the runtime to use; valid options are 'llama.cpp' and 'vllm'",
+        choices=["llama.cpp", "vllm", "mlx"],
+        help="specify the runtime to use; valid options are 'llama.cpp', 'vllm', and 'mlx'",
     )
     parser.add_argument(
         "--store",
         default=CONFIG.store,
+        type=abspath,
         help="store AI Models in the specified directory",
     )
     parser.add_argument(
@@ -270,6 +275,12 @@ def post_parse_setup(args):
     if hasattr(args, "runtime_args"):
         args.runtime_args = shlex.split(args.runtime_args)
 
+    # MLX runtime automatically requires --nocontainer
+    if getattr(args, "runtime", None) == "mlx":
+        if getattr(args, "container", None) is True:
+            logger.info("MLX runtime automatically uses --nocontainer mode")
+        args.container = False
+
     configure_logger("DEBUG" if args.debug else "WARNING")
 
 
@@ -371,7 +382,7 @@ def list_files_by_modification(args):
         if os.path.exists(path):
             models.append(path)
         else:
-            print(f"Broken symlink found in: {args.store}/models/{path} \nAttempting removal")
+            perror(f"Broken symlink found in: {args.store}/models/{path} \nAttempting removal")
             New(str(path).replace("/", "://", 1), args).remove(args)
 
     return sorted(models, key=lambda p: os.path.getmtime(p), reverse=True)
@@ -904,10 +915,16 @@ def chat_parser(subparsers):
         choices=['never', 'always', 'auto'],
         help='possible values are "never", "always" and "auto".',
     )
+    parser.add_argument(
+        "--list",
+        "--ls",
+        action="store_true",
+        help="list the available models at an endpoint",
+    )
     parser.add_argument("--prefix", type=str, help="prefix for the user prompt", default=default_prefix())
     parser.add_argument("--rag", type=str, help="a file or directory to use as context for the chat")
     parser.add_argument("--url", type=str, default="http://127.0.0.1:8080/v1", help="the url to send requests to")
-    parser.add_argument("MODEL", completer=local_models)  # positional argument
+    parser.add_argument("--model", "-m", type=str, completer=local_models, help="model for inferencing")
     parser.add_argument(
         "ARGS", nargs="*", help="overrides the default prompt, and the output is returned without entering the chatbot"
     )
@@ -1173,17 +1190,17 @@ def main():
         perror("Error: " + str(e).strip("'\""))
         sys.exit(exit_code)
 
+    if not args.subcommand:
+        parser.print_usage()
+        perror("ramalama: requires a subcommand")
+        return 0
+
     try:
         args.func(args)
     except urllib.error.HTTPError as e:
         eprint(f"pulling {e.geturl()} failed: {e}", errno.EINVAL)
     except HelpException:
         parser.print_help()
-    except AttributeError as e:
-        parser.print_usage()
-        perror("ramalama: requires a subcommand")
-        if getattr(args, "debug", False):
-            raise e
     except IndexError as e:
         eprint(e, errno.EINVAL)
     except KeyError as e:
