@@ -7,7 +7,6 @@ import sys
 # Live reference for checking global vars
 import ramalama.common
 from ramalama.common import check_nvidia, exec_cmd, get_accel_env_vars, perror, run_cmd
-from ramalama.console import EMOJI
 from ramalama.logger import logger
 
 
@@ -31,7 +30,6 @@ class Engine:
         self.add_privileged_options()
         self.add_pull_newer()
         self.add_rag()
-        self.add_subcommand_env()
         self.add_tty_option()
         self.handle_podman_specifics()
         self.add_detach_option()
@@ -49,8 +47,8 @@ class Engine:
         if not self.args.dryrun and self.use_docker and self.args.pull == "newer":
             try:
                 if not self.args.quiet:
-                    print(f"Checking for newer image {self.args.image}")
-                run_cmd([self.args.engine, "pull", "-q", self.args.image], ignore_all=True)
+                    perror(f"Checking for newer image {self.args.image}")
+                run_cmd([str(self.args.engine), "pull", "-q", self.args.image], ignore_all=True)
             except Exception:  # Ignore errors, the run command will handle it.
                 pass
         else:
@@ -91,14 +89,7 @@ class Engine:
             return False
         if getattr(self.args, "ARGS", None):
             return False
-        return getattr(self.args, "subcommand", "") != "run"
-
-    def add_subcommand_env(self):
-        if EMOJI and self.use_tty():
-            if os.path.basename(self.args.engine) == "podman":
-                self.exec_args += ["--env", "LLAMA_PROMPT_PREFIX=🦭 > "]
-            if self.use_docker:
-                self.exec_args += ["--env", "LLAMA_PROMPT_PREFIX=🐋 > "]
+        return getattr(self.args, "subcommand", "") == "run"
 
     def add_env_option(self):
         for env in getattr(self.args, "env", []):
@@ -116,10 +107,12 @@ class Engine:
         if getattr(self.args, "port", "") == "":
             return
 
+        host = getattr(self.args, "host", "0.0.0.0")
+        host = f"{host}:" if host != "0.0.0.0" else ""
         if self.args.port.count(":") > 0:
-            self.exec_args += ["-p", self.args.port]
+            self.exec_args += ["-p", f"{host}{self.args.port}"]
         else:
-            self.exec_args += ["-p", f"{self.args.port}:{self.args.port}"]
+            self.exec_args += ["-p", f"{host}{self.args.port}:{self.args.port}"]
 
     def add_device_options(self):
         if getattr(self.args, "device", None):
@@ -186,7 +179,7 @@ def dry_run(args):
 
 
 def images(args):
-    conman = args.engine
+    conman = str(args.engine) if args.engine is not None else None
     if conman == "" or conman is None:
         raise ValueError("no container manager (Podman, Docker) found")
 
@@ -211,7 +204,7 @@ def images(args):
 
 
 def containers(args):
-    conman = args.engine
+    conman = str(args.engine) if args.engine is not None else None
     if conman == "" or conman is None:
         raise ValueError("no container manager (Podman, Docker) found")
 
@@ -236,8 +229,8 @@ def containers(args):
 
 
 def info(args):
-    conman = args.engine
-    if conman == "":
+    conman = str(args.engine) if args.engine is not None else None
+    if conman == "" or conman is None:
         raise ValueError("no container manager (Podman, Docker) found")
 
     conman_args = [conman, "info", "--format", "json"]
@@ -253,8 +246,8 @@ def info(args):
 def inspect(args, name, format=None, ignore_stderr=False):
     if not name:
         raise ValueError("must specify a container name")
-    conman = args.engine
-    if conman == "":
+    conman = str(args.engine) if args.engine is not None else None
+    if conman == "" or conman is None:
         raise ValueError("no container manager (Podman, Docker) found")
 
     conman_args = [conman, "inspect"]
@@ -268,8 +261,8 @@ def inspect(args, name, format=None, ignore_stderr=False):
 def stop_container(args, name):
     if not name:
         raise ValueError("must specify a container name")
-    conman = args.engine
-    if conman == "":
+    conman = str(args.engine) if args.engine is not None else None
+    if conman == "" or conman is None:
         raise ValueError("no container manager (Podman, Docker) found")
 
     ignore_stderr = False
@@ -310,12 +303,12 @@ def container_connection(args, name, port):
     if not port:
         raise ValueError("must specify a port to check")
 
-    conman = args.engine
-    if conman == "":
+    conman = str(args.engine) if args.engine is not None else None
+    if conman == "" or conman is None:
         raise ValueError("no container manager (Podman, Docker) found")
 
     conman_args = [conman, "port", name, port]
-    output = run_cmd(conman_args, debug=args.debug).stdout.decode("utf-8").strip()
+    output = run_cmd(conman_args).stdout.decode("utf-8").strip()
     return "" if output == "" else output.split(">")[-1].strip()
 
 
